@@ -5,8 +5,14 @@ from collections import defaultdict
 import random
 import numpy as np
 import math
+import xmltodict
 
-# from lxml import etree
+
+def xml_to_dict(fname):
+    """Parse XML file."""
+    with open(fname, "r") as f:
+        data = xmltodict.parse(f.read())
+    return data
 
 
 def get_counts(array=["W", "W", "Mo", "Mo", "S", "S"]):
@@ -137,6 +143,14 @@ def array_to_string(arr=[]):
     return ",".join(map(str, arr))
 
 
+def chunks(lst, n):
+    """Split successive n-sized chunks from list."""
+    x = []
+    for i in range(0, len(lst), n):
+        x.append(lst[i : i + n])
+    return x
+
+
 def check_match(a, b, tol=1e-8):
     """Check if a and b are the same, taking into account PBCs."""
     if abs(a[0] - b[0]) < tol or abs(abs(a[0] - b[0]) - 1) < tol:
